//===========================================================
// MenuEdit - Generic Win16/Win32 template
//
// Copyright (C) 1995 Ziff-Davis Publishing Company
// First published in PC Magazine by Douglas Boling
//
// Revision History:
//
// 1.0   Initial Release
//
//===========================================================

#define    APPNAME        "MenuEdit"

//-----------------------------------------------------------
// Include files
//-----------------------------------------------------------
#include "windows.h"
#include "commdlg.h"
#include "shellapi.h"

#include "shlobj.h"

// Use different include file for Win32
#if (WIN32)
#include "winver.h"
#else
#include "ver.h"
#endif

#include "stdlib.h"
#include "memory.h"

#include "MenuEdit.h"
#include "Resource.h"


typedef struct {
	BOOL fUseDDE;
	char szFileType[128];
	char szAction[64];
	char szCommand[256];
	char szDDEMsg[256];
	char szDDEApp[256];
	char szDDEAppNR[256];
	char szDDETopic[256];
	char szOthers[256];
} ADDEDITSTRUCT;
typedef ADDEDITSTRUCT far *LPADDEDITSTRUCT;

INT GetSelectedMenuItem (HWND, char *, char *, INT);
INT GetSelectedType (HWND, char *, INT, char **);
INT GetKeyFromType (LPSTR, LPSTR *);
INT FillFileTypeBox (HWND);
INT FillFileMenuBox (HWND, LPSTR);
INT FillAddEditStruct (char *, LPADDEDITSTRUCT);
INT SaveMenuRegStuff (char *, LPADDEDITSTRUCT, BOOL);
//-----------------------------------------------------------
// Global data
//-----------------------------------------------------------
// Message dispatch table for MainWindowProc
struct decodeUINT MainMessages[] = {
	WM_CREATE, DoCreateMain,
	WM_SHOWWINDOW, DoShowWindowMain,
	WM_COMMAND, DoCommandMain,
	WM_DESTROY, DoDestroyMain,
};
// Command Message dispatch for MainWindowProc
struct decodeCMD MainCtlItems[] = {
	IDD_ALLFILES, DoMainCtlAllFiles,
	IDD_UNKNOWN, DoMainCtlUnknown,
	IDD_SPECIFICTYPES, DoMainCtlTypeFiles,
	IDD_TYPELIST, DoMainCtlTypeList,
	IDD_MENULIST, DoMainCtlMenuList,
	IDD_NEW, DoMainCtlNew,
	IDD_EDIT, DoMainCtlOpen,
	IDD_DELETE, DoMainCtlCut,
	IDD_SETDEFAULT, DoMainCtlSetDefault,
	IDD_EXIT, DoMainCtlExit,
	IDD_ABOUT, DoMainCtlAbout,
};
char szAppName[] = APPNAME;               // Application name
char szTitleText[] = APPNAME": Untitled"; // Application window title
char szMenuName[] = APPNAME;              // Menu name
char szIconName[] = APPNAME;              // Icon name
char szProfileName[MAX_PATH];             // INI file name

HANDLE hInst;
HWND hMain;
HINSTANCE h3DLib = 0;                      // Used under Win16	
char szDebug[256];
BOOL fFirst = TRUE, fListLast = FALSE;
INT sTypeSel = 0;

//============================================================
// WinMain -- entry point for this application from Windows.
//============================================================
INT APIENTRY WinMain(HANDLE hInstance, HANDLE hPrevInstance, 
                     LPSTR lpCmdLine, INT nCmdShow) {
	HACCEL hAccel;	
	MSG	msg;
	INT	rc;

	hInst = hInstance;

   if(!hPrevInstance) 
		if((rc = InitApp(hInstance)) != 0)
			return rc;

	// Initialize this instance
	if((rc = InitInstance(hInstance, lpCmdLine, nCmdShow)) != 0)
 	return rc;

	hAccel = LoadAccelerators (hInstance, szAppName);
	//
	// Application message loop
	//
	while (GetMessage (&msg, NULL, 0, 0)) {
		if (!IsDialogMessage (hMain, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
/*
	while (GetMessage (&msg, NULL, 0, 0)) {
		if ((!IsDialogMessage (hMain, &msg)) && (!TranslateAccelerator (hMain, hAccel, &msg))) {
		    
//		if (!IsDialogMessage (hMain, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
*/
	// Instance cleanup
	return TermInstance(hInstance, msg.wParam);
}



//-----------------------------------------------------------
// InitApp - Global initialization code for this application.
//-----------------------------------------------------------
INT InitApp(HANDLE hInstance) {
	WNDCLASS 	wc;
	//
	// Register App Main Window class
	//		
	wc.style = 0;                             // Window style
	wc.lpfnWndProc = MainWndProc;             // Callback function
	wc.cbClsExtra = 0;                        // Extra class data
	wc.cbWndExtra = DLGWINDOWEXTRA;           // Extra window data
	wc.hInstance = hInstance;                 // Owner handle
	wc.hIcon = LoadIcon(hInst, szIconName);   // Application icon
	wc.hCursor = LoadCursor(NULL, IDC_ARROW); // Default cursor
	wc.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
	wc.lpszMenuName =  szMenuName;            // Menu name
	wc.lpszClassName = szAppName;             // Window class name
	if (MyRegisterClass(&wc) == 0)
		return 1;

	return 0;
}
//-----------------------------------------------------------
// InitInstance - Instance initialization code for this app.
//-----------------------------------------------------------
INT InitInstance(HANDLE hInstance, LPSTR lpCmdLine, INT nCmdShow) {
	INT x, y;
	DWORD fFlags = 0;
	
	//Create INI filename
	GetModuleFileName (hInst, szProfileName, sizeof (szProfileName));
	for (x = lstrlen (szProfileName); x && szProfileName[x] != '.'; --x);
	lstrcpy (&szProfileName[x], ".INI");

	// If 3DLib present, load it for 3d effects.
	MyCtl3dStuff (&h3DLib, hInstance, CTL3D_INIT);

	//Read INI file info
	x = GetPrivateProfileInt (szAppName, PRO_XPOS, CW_USEDEFAULT,
	                          szProfileName);
	y = GetPrivateProfileInt (szAppName, PRO_YPOS, CW_USEDEFAULT,
	                          szProfileName);
	// Create main window
	hMain = CreateDialog (hInstance, szAppName, 0, NULL);
	if(!hMain) return 0x10;
	
	if (!nCmdShow)
		ShowWindow(hMain, SW_SHOW);
	else if (fFlags & SFLAG_PRINTONLY) 
		ShowWindow(hMain, SW_SHOWMINIMIZED);
	else	
		ShowWindow(hMain, nCmdShow);
	UpdateWindow(hMain);              // force WM_PAINT message
	return 0;                         // return success flag
}
//------------------------------------------------------------
// TermInstance - Instance termination code for this app.
//------------------------------------------------------------
INT TermInstance(HANDLE hInstance, int sDefRC) {

	// Unregister 3d-lib
	MyCtl3dStuff (&h3DLib, hInstance, CTL3D_TERM);

	return sDefRC;
}
//============================================================
// Message handling procedures for MainWindow
//============================================================
//------------------------------------------------------------
// MainWndProc - Callback function for application window
//------------------------------------------------------------
LONG CALLBACK MainWndProc(HWND hWnd, UINT wMsg, UINT wParam, 
                          LONG lParam) {
	INT i;
	//
	// Search message list to see if we need to handle this
	// message.  If in list, call procedure.
	//
	for(i = 0; i < dim(MainMessages); i++) {
		if(wMsg == MainMessages[i].Code) 
			return (*MainMessages[i].Fxn)(hWnd, wMsg, wParam, lParam);
	}
	return DefWindowProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoCreateMain - process WM_CREATE message for frame window.
//------------------------------------------------------------ 
LONG DoCreateMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {

	return 0;
}
//------------------------------------------------------------
// DoShowWindowMain - process WM_SHOWWINDOW message for frame window.
//------------------------------------------------------------ 
LONG DoShowWindowMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	INT sTabs[3];

	if (fFirst) {
		// Set tabstop for type listbox.  Tab used to hide data
		// to the right of the right edge.

		// First, the file type box.
		sTabs[0] = 200;
		SendDlgItemMessage (hWnd, IDD_TYPELIST, LB_SETTABSTOPS, 1,
			                (LPARAM)(LPINT)sTabs);

		// Now the menu item list box.
		sTabs[0] = 75;
		sTabs[1] = 200;
		SendDlgItemMessage (hWnd, IDD_MENULIST, LB_SETTABSTOPS, 2,
			                (LPARAM)(LPINT)sTabs);

		FillFileTypeBox (GetDlgItem (hWnd, IDD_TYPELIST));
		CheckRadioButton (hWnd, IDD_ALLFILES, IDD_SPECIFICTYPES, IDD_ALLFILES);
		PostWm_CommandMsg (hWnd, IDD_ALLFILES, BN_CLICKED, 0);
		fFirst = FALSE;
	}

	return 0;
}
//------------------------------------------------------------
// DoCommandMain - process WM_COMMAND message for frame window 
// by decoding the menubar item with the menuitems[] array, 
// then running the corresponding function to process the command.
//------------------------------------------------------------ 
LONG DoCommandMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
	INT	i;
	UINT	idItem, wNotifyCode;
	HWND	hwndCtl;
	
#if (WIN32)
	idItem = (UINT) LOWORD(wParam);
	wNotifyCode = (UINT) HIWORD(wParam);
	hwndCtl = (HWND) lParam;
#else
	idItem = (UINT) wParam;
	hwndCtl = (HWND) LOWORD(lParam);
	wNotifyCode = (UINT) HIWORD(lParam);
#endif

	//
	// Call routine to handle control message
	//
	for(i = 0; i < dim(MainCtlItems); i++) {
		if(idItem == MainCtlItems[i].Code)
			return (*MainCtlItems[i].Fxn)(hWnd, idItem, hwndCtl, 
			                               wNotifyCode);
	}
	return DefWindowProc(hWnd, wMsg, wParam, lParam);
}
//------------------------------------------------------------
// DoDestroyMain - process WM_DESTROY message for frame window.
//------------------------------------------------------------ 
LONG DoDestroyMain (HWND hWnd, UINT wMsg, UINT wParam, LONG lParam) {
   RECT	rect;

	//Save Window position
	if	(!IsIconic (hWnd) && !IsZoomed (hWnd)) {
		GetWindowRect (hWnd, &rect);
		MyWritePrivateProfileInt (szAppName, PRO_XPOS, rect.left,
		                          szProfileName);
		MyWritePrivateProfileInt (szAppName, PRO_YPOS, rect.top,
		                          szProfileName);
	}
	PostQuitMessage (0);
	return 0;
}
//============================================================
// Control handling procedures for MainWindow
//============================================================
//------------------------------------------------------------
// DoMainCtlAllFiles - Process All Files button
//------------------------------------------------------------ 
LONG DoMainCtlAllFiles (HWND hWnd, UINT idItem, HWND hwndCtl, 
                        UINT wNotifyCode) {
	
	if (wNotifyCode == BN_CLICKED) {


		if (fListLast) {
			sTypeSel = SendDlgItemMessage (hWnd, IDD_TYPELIST, 
				                           LB_GETCURSEL, 0, 0);
			SendDlgItemMessage (hWnd, IDD_TYPELIST, 
			                    LB_SETCURSEL, (WPARAM)-1, 0);
			fListLast = FALSE;
		}
		PostWm_CommandMsg (hWnd, IDD_TYPELIST, LBN_SELCHANGE, 0);
		EnableWindow (GetDlgItem (hWnd, IDD_TYPELIST), FALSE);
	}
	return 0;
}
//------------------------------------------------------------
// DoMainCtlUnknown - Process Unknown files button
//------------------------------------------------------------ 
LONG DoMainCtlUnknown (HWND hWnd, UINT idItem, HWND hwndCtl, 
                       UINT wNotifyCode) {

	if (wNotifyCode == BN_CLICKED) {
		if (fListLast) {
			sTypeSel = SendDlgItemMessage (hWnd, IDD_TYPELIST, 
				                           LB_GETCURSEL, 0, 0);
			SendDlgItemMessage (hWnd, IDD_TYPELIST, 
				                           LB_SETCURSEL, (WPARAM)-1, 0);
			fListLast = FALSE;
		}
		PostWm_CommandMsg (hWnd, IDD_TYPELIST, LBN_SELCHANGE, 0);
		EnableWindow (GetDlgItem (hWnd, IDD_TYPELIST), FALSE);
	}
	return 0;
}
//------------------------------------------------------------
// DoMainCtlTypeFiles - Process Specific type button
//------------------------------------------------------------ 
LONG DoMainCtlTypeFiles (HWND hWnd, UINT idItem, HWND hwndCtl, 
                         UINT wNotifyCode) {

	if (wNotifyCode == BN_CLICKED) {
		fListLast = TRUE;
		SendDlgItemMessage (hWnd, IDD_TYPELIST, 
		                    LB_SETCURSEL, sTypeSel, 0);
		EnableWindow (GetDlgItem (hWnd, IDD_TYPELIST), TRUE);
		PostWm_CommandMsg (hWnd, IDD_TYPELIST, LBN_SELCHANGE, 0);
	}
	return 0;
}
//------------------------------------------------------------
// DoMainCtlTypeList - Process Type List Ctl item    
//------------------------------------------------------------ 
LONG DoMainCtlTypeList (HWND hWnd, UINT idItem, HWND hwndCtl, 
                        UINT wNotifyCode) {
	char szItem[256], *pszExtLst;
	BOOL fEnable, fNewEnable, fApp;

	if (wNotifyCode == LBN_SELCHANGE) {
		fNewEnable = FALSE;
		fEnable = FALSE;
		fApp = FALSE;
		if (GetSelectedType (hWnd, szItem, sizeof (szItem), &pszExtLst)) {
			if (pszExtLst) {
				if (lstrcmpi (pszExtLst, ".exe") == 0)
					fApp = TRUE;
				SetDlgItemText (hWnd, IDD_EXTLIST, pszExtLst);
			} else
				SetDlgItemText (hWnd, IDD_EXTLIST, "");

			fNewEnable = TRUE;			
			FillFileMenuBox (GetDlgItem (hWnd, IDD_MENULIST), szItem);
			// Enable the proper buttons
			if (SendDlgItemMessage (hWnd, IDD_MENULIST, LB_GETCOUNT, 0, 0)) {
				fEnable = TRUE;
				SendDlgItemMessage (hWnd, IDD_MENULIST, LB_SETCURSEL, 0, 0);
			}
		} else
			SendDlgItemMessage (hWnd, IDD_MENULIST, LB_RESETCONTENT, 0, 0);

		EnableWindow (GetDlgItem (hWnd, IDD_NEW), fNewEnable);
		EnableWindow (GetDlgItem (hWnd, IDD_EDIT), fEnable);
		EnableWindow (GetDlgItem (hWnd, IDD_DELETE), fEnable);
		// Hack to prevent All failes from enabling default btn
		if (lstrcmp (szItem, "All Files") == 0)
			fEnable = FALSE;
		// Hack to prevent users from changing the default for apps (EXE files)
		if (fApp)
			fEnable = FALSE;
		EnableWindow (GetDlgItem (hWnd, IDD_SETDEFAULT), fEnable);
		PostWm_CommandMsg (hWnd, IDD_MENULIST, LBN_SELCHANGE, 0);
	}
	return 0;
}
//------------------------------------------------------------
// DoMainCtlMenuList - Process MenuList Ctl item
//------------------------------------------------------------ 
LONG DoMainCtlMenuList (HWND hWnd, UINT idItem, HWND hwndCtl, 
                        UINT wNotifyCode) {
	char szItem[256], *pszExtLst;
	BOOL fEnable;

	fEnable = FALSE;

	// Hack to prevent users from editing or removing Open command
	// for apps.
	if ((wNotifyCode == LBN_DBLCLK) || (wNotifyCode == LBN_SELCHANGE)) {
		if (GetSelectedType (hWnd, szItem, sizeof (szItem), &pszExtLst)) {
			if (pszExtLst) {
				if (lstrcmpi (pszExtLst, ".exe") == 0) {
					GetSelectedMenuItem (hWnd, szItem, 0, 0);
					if (lstrcmpi (szItem, "exefile\\shell\\open") == 0) 
						fEnable = FALSE;
					else 
						fEnable = TRUE;
					EnableWindow (GetDlgItem (hWnd, IDD_EDIT), fEnable);
					EnableWindow (GetDlgItem (hWnd, IDD_DELETE), fEnable);
				} else
					fEnable = TRUE;
			} else if (lstrlen (szItem))
				fEnable = TRUE;
		}
	}
	if ((wNotifyCode == LBN_DBLCLK) && fEnable)
		PostWm_CommandMsg (hWnd, IDD_EDIT, BN_CLICKED, 0);

	return 0;
}
//------------------------------------------------------------
// DoMainCtlNew - Process New Ctl item
//------------------------------------------------------------ 
LONG DoMainCtlNew (HWND hWnd, UINT idItem, HWND hwndCtl, 
                    UINT wNotifyCode) {
	INT i;
	char szItem[256];
	LPSTR pszKey;
	ADDEDITSTRUCT aes;

	memset (&aes, 0, sizeof (ADDEDITSTRUCT));

	// Get current selection 
	GetSelectedType (hWnd, szItem, sizeof (szItem), 0);
	GetKeyFromType (szItem, &pszKey);
	lstrcpy (aes.szFileType, szItem);
	// Use this function to get the 'others' list.
	SendDlgItemMessage (hWnd, IDD_MENULIST, LB_SETCURSEL, (WPARAM)-1, 0);
	GetSelectedMenuItem (hWnd, szItem, aes.szOthers, sizeof (aes.szOthers));
	
	i = MyDisplayDialog(hInst, "AddEditDlg", hWnd, 
	                    (WNDPROC) AddEditDlgProc, (LPARAM)(LPVOID)&aes);
	if (i == 1) {
		SaveMenuRegStuff (pszKey, &aes, TRUE);
		PostWm_CommandMsg (hWnd, IDD_TYPELIST, LBN_SELCHANGE, 0);
	}
	return 0;
}
//------------------------------------------------------------
// DoMainCtlOpen - Process Open Ctl item
//------------------------------------------------------------ 
LONG DoMainCtlOpen (HWND hWnd, UINT idItem, HWND hwndCtl, 
                     UINT wNotifyCode) {
	INT i;
	char szKey[256];
	LPSTR pszKey;
	ADDEDITSTRUCT aes;

	memset (&aes, 0, sizeof (ADDEDITSTRUCT));
	// Get current selection 
	GetSelectedType (hWnd, szKey, sizeof (szKey), 0);
	GetKeyFromType (szKey, &pszKey);
	lstrcpy (aes.szFileType, szKey);
	
	if (GetSelectedMenuItem (hWnd, szKey, aes.szOthers, sizeof (aes.szOthers))) {
		if (FillAddEditStruct (szKey, &aes) == 0) {
	
			i = MyDisplayDialog(hInst, "AddEditDlg", hWnd, 
			                    (WNDPROC) AddEditDlgProc, (LPARAM)(LPVOID)&aes);
			if (i == 1) {
				SaveMenuRegStuff (szKey, &aes, FALSE);
				PostWm_CommandMsg (hWnd, IDD_TYPELIST, LBN_SELCHANGE, 0);
			}
		}
	}
	return 0;
}
//------------------------------------------------------------
// DoMainCtlCut - Process Cut Ctl item
//------------------------------------------------------------ 
LONG DoMainCtlCut (HWND hWnd, UINT idItem, HWND hwndCtl, 
                   UINT wNotifyCode) {
	char szKey[256], szText[80];
	LONG lRC;

	if (GetSelectedMenuItem (hWnd, szKey, 0, 0)) {

		wsprintf (szText, "Are you sure you want to delete the menu item?");

		lRC = MessageBox (hWnd, szText, APPNAME, MB_YESNO | MB_ICONQUESTION);
		if (lRC != IDYES)
			return 0;

		RegDeleteKey (HKEY_CLASSES_ROOT, szKey);
		PostWm_CommandMsg (hWnd, IDD_TYPELIST, LBN_SELCHANGE, 0);
	}
	return 0;
}
//------------------------------------------------------------
// DoMainCtlSetDefault - Process Set Defaule Ctl item
//------------------------------------------------------------ 
LONG DoMainCtlSetDefault (HWND hWnd, UINT idItem, HWND hwndCtl, 
                          UINT wNotifyCode) {
	char szKey[256], *pszTemp;

	if (GetSelectedMenuItem (hWnd, szKey, 0, 0)) {
		// Get ptr to menu name
		pszTemp = strrchr (szKey, '\\');
		if (pszTemp) {
			*pszTemp++ = '\0';
			// Set the default value of the shell keyword to the new
			// default menu item.
			RegSetValue (HKEY_CLASSES_ROOT, szKey, REG_SZ,
			             pszTemp, lstrlen (pszTemp));
			// Refresh the menu
			PostWm_CommandMsg (hWnd, IDD_TYPELIST, LBN_SELCHANGE, 0);
		}
	}
	return 0;
}
//------------------------------------------------------------
// DoMainCtlExit - Process Exit Ctl item
//------------------------------------------------------------ 
LONG DoMainCtlExit (HWND hWnd, UINT idItem, HWND hwndCtl, 
                     UINT wNotifyCode) {

	// Close the app
	SendMessage (hWnd, WM_CLOSE, 0, 0);
	return 0;
}
//------------------------------------------------------------
// DoMainCtlAbout - Process About Ctl item
//------------------------------------------------------------ 
LONG DoMainCtlAbout (HWND hWnd, UINT idItem, HWND hwndCtl, 
                     UINT wNotifyCode) {

	MyDisplayDialog(hInst, "AboutBox", hWnd, 
   	             (WNDPROC) AboutDlgProc, 0);

	return 0;
}
//============================================================
// Program Specific routines
//============================================================
//------------------------------------------------------------
// FillFileTypeBox - Fills the file type list box
//------------------------------------------------------------
INT FillFileTypeBox (HWND hWnd) {
	LONG lSize, lVSize, lRC;
	HKEY hKey = 0;
	INT i, j, sIndex;
	char szKey[256], szValue[256], szTypeKey[64], szExt[64];
	FILETIME ftTime;

	// Clear listbox
	SendMessage (hWnd, LB_RESETCONTENT, 0, 0);
	for (i = 0; ;i++) {
		lSize = sizeof (szKey);
		lVSize = sizeof (szValue);
		// Enumerate each key.
		lRC = RegEnumKeyEx(HKEY_CLASSES_ROOT, i, szKey, &lSize, NULL,
		                   szValue, &lVSize, &ftTime);
		if (lRC != ERROR_SUCCESS)
			break;
		// Find all keys that are extensions
		if (szKey[0] == '.') {
			if (lstrcmp (szKey, ".aab") == 0)
				lVSize = 0;

			lstrcpyn (szTypeKey, szKey, sizeof (szTypeKey));
			lstrcpyn (szExt, szKey, sizeof (szTypeKey));
			// Get extension description
			lVSize = sizeof (szValue);
			lRC = RegQueryValue (HKEY_CLASSES_ROOT, szKey, szValue, &lVSize);
			if (lRC == ERROR_SUCCESS) {

				// Screen out CLSID strings
				if (lVSize > 7) {
					if (szValue[5] == '\\') {
						char ch = szValue[6];
						szValue[6] = '\0';
						if (lstrcmp (szValue, "CLSID\\") == 0) 
							continue;
						szValue[6] = ch;
					} else if ((szValue[0] == '{') && (szValue[lstrlen (szValue)-1] == '}')) 
						continue;
				}
				// Make sure there isn't a blank description key
				if (lVSize > 1) {
					lstrcpyn (szTypeKey, szValue, sizeof (szTypeKey));
					lstrcpy (szKey, szValue);
					// Get text description
					lVSize = sizeof (szValue);
					lRC = RegQueryValue (HKEY_CLASSES_ROOT, szKey, szValue, &lVSize);
				}
			} else 
				szValue[0] = '\0';

			// For extensions without descriptions, create one.
			if (lstrlen (szValue) == 0) {
				if (szKey[0] == '.')
					for (j = 0; j < lstrlen (szKey); j++) 
						szKey[j] = szKey[j+1];
				CharUpper (szKey);
				lstrcat (szKey, " File");
				lstrcpy (szValue, szKey);
			}
			lstrcat (szValue, "\t");
			lstrcat (szValue, szTypeKey);
			// See if file type already in list.  If not add.
			sIndex = SendMessage (hWnd, LB_FINDSTRING, (UINT)-1, (LPARAM)(LPSTR)szValue);
			if (sIndex == LB_ERR) {
				lstrcat (szValue, "\t");
				lstrcat (szValue, szExt);
				SendMessage (hWnd, LB_ADDSTRING, 0, (LPARAM)(LPSTR)szValue);
			} else {
				// If type in list, append new extension to list item text
				j = SendMessage (hWnd, LB_GETTEXTLEN, sIndex, 0);
				if (j + lstrlen (szExt) + 2 < sizeof (szValue)) {
					SendMessage (hWnd, LB_GETTEXT, sIndex, (LPARAM)(LPSTR)szValue);
					lstrcat (szValue, ", ");
					lstrcat (szValue, szExt);
					SendMessage (hWnd, LB_DELETESTRING, sIndex, 0);
					SendMessage (hWnd, LB_INSERTSTRING, sIndex, (LPARAM)(LPSTR)szValue);
				}
			}
		}
	}
	return 0;
}
//------------------------------------------------------------
// GetSelectedMenuItem - Returns the current selected menu item
//------------------------------------------------------------
INT GetSelectedMenuItem (HWND hWnd, char *szKey, char *pszOthers, int sOMax) { 
	INT i, sSel, sCnt, sLen;
	char szTemp[384], *pszTemp;

	*szKey = '\0';
	sCnt = SendDlgItemMessage (hWnd, IDD_MENULIST, LB_GETCOUNT, 0, 0);
	sSel = SendDlgItemMessage (hWnd, IDD_MENULIST, LB_GETCURSEL, 0, 0);
	if (sSel == LB_ERR) 
		sSel = -1;
	for (i = 0; i < sCnt; i++) {
		sLen = SendDlgItemMessage (hWnd, IDD_MENULIST, LB_GETTEXTLEN, i,0);
		if (sLen < sizeof (szTemp)) 
			SendDlgItemMessage (hWnd, IDD_MENULIST, LB_GETTEXT, i,(LPARAM)(LPSTR)szTemp);
		else
			szTemp[0] = '\0';

		szTemp[sizeof (szTemp)-1] = '\0';
		if (i == sSel) {

			// Skip past default string
			pszTemp = strrchr (szTemp, '\t');
			if (pszTemp == 0) return 0;
			lstrcpy (szKey, pszTemp+1);
		} else if (pszOthers) {
			// All other menu items are added to others list.

			// Find end of menu name
			pszTemp = strchr (szTemp, '\t');
			if (pszTemp == 0) return 0;
			*pszTemp = '\0';
			sLen = lstrlen (szTemp) + 1;
			if (sLen < sOMax) {
				lstrcpy (pszOthers, szTemp);
				pszOthers += sLen;
				sOMax -= sLen;
			}
		}
	}
	// Insure termination of others list.  If no room, truncate last entry.
	if (pszOthers) {
		if (sOMax > 1)
			*(LPINT)pszOthers = 0;
		else
			*((LPINT)pszOthers-1) = 0;
	}
	return lstrlen (szKey);
}
//------------------------------------------------------------
// GetSelectedType - Returns the current selected file type
//------------------------------------------------------------
INT GetSelectedType (HWND hWnd, char *szItem, INT sLen, char **ppszExtLst) { 
	INT i, sTextLen;
	char *pszExt = 0;

	*szItem = '\0';
	// Get current selection 
	if (IsDlgButtonChecked(hWnd, IDD_ALLFILES) == 1) {
		lstrcpy (szItem, "All Files\t*");

	} else if (IsDlgButtonChecked(hWnd, IDD_UNKNOWN) == 1) {
		lstrcpy (szItem, "Unknown Files\tUnknown");

	} else {
		i = SendDlgItemMessage (hWnd, IDD_TYPELIST, LB_GETCURSEL, 0, 0);
		if (i != LB_ERR) {
			sTextLen = SendDlgItemMessage (hWnd, IDD_TYPELIST, LB_GETTEXTLEN, i, 0);
			if (sLen > sTextLen) {
				SendDlgItemMessage (hWnd, IDD_TYPELIST, LB_GETTEXT, i, 
					                (LPARAM)(LPSTR)szItem);
				pszExt = strrchr (szItem, '\t');
				if (pszExt)
					*pszExt++ = '\0';
			} else
				memset (szItem, 0, sizeof (szItem));
		}
	}
	if (ppszExtLst)
		*ppszExtLst = pszExt;
	return lstrlen (szItem);
}
//------------------------------------------------------------
// GetKeyFromType - Parses key from file type string.
//------------------------------------------------------------
INT GetKeyFromType (LPSTR szFileDesc, LPSTR *ppszKey) {
	INT i;

	// Parse file description to get reg key inserted after tab char.
	for (i = 0; (szFileDesc[i] != '\0') && (szFileDesc[i] != '\t');i++);

	if (szFileDesc[i] == '\0')
		i = 0;
	else
		szFileDesc[i++] = '\0';

	*ppszKey = &szFileDesc[i];
	lstrcat (*ppszKey, "\\Shell");

	return lstrlen (*ppszKey);
}
//------------------------------------------------------------
// FillFileMenuBox - Fills the file menu list box
//------------------------------------------------------------
INT FillFileMenuBox (HWND hWnd, LPSTR szFileDesc) {
	LONG lSize, lRC, lType;
	HKEY hKey = 0;
	INT i, sListBoxCnt, sDefSet = 0, sDefItem;
	char szDefault[128], szMenu[128], szItem[256];
	LPSTR lpszKey;
	FILETIME ftTime;

	// Clear listbox
	SendMessage (hWnd, LB_RESETCONTENT, 0, 0);
	// Parse key string from listbox item text
	GetKeyFromType (szFileDesc, &lpszKey);

	if (RegOpenKeyEx (HKEY_CLASSES_ROOT, lpszKey, 0, (REGSAM)KEY_READ, &hKey) == ERROR_SUCCESS) {

		// Get default menu item and menu order if specified.
		lSize = sizeof (szDefault);
		lRC = RegQueryValueEx (hKey, "", NULL, &lType, (LPBYTE)szDefault, &lSize);
		if ((lRC != ERROR_SUCCESS) || (lSize < 2))
			lstrcpy (szDefault, "open");
		else {
			char *pszDefault;
			pszDefault = szDefault;
			// If default string specifies order, term string after 1st to get
			// default.  
			while (*pszDefault) {
				if (*pszDefault == ',')
					*pszDefault = '\0';
				pszDefault++;
			}
			*(pszDefault+1) = '\0';
		}
			
		// Enumerate each menu command
		for (i = 0, sListBoxCnt = 0; ;i++) {
			// Get menu items
			lSize = sizeof (szMenu);
			lRC = RegEnumKeyEx(hKey, i, szMenu, &lSize, NULL, NULL, NULL, &ftTime);
			if (lRC != ERROR_SUCCESS)
				break;
			// Don't display PrintTo menu item per Win95 guidelines.
			if (lstrcmpi (szMenu, "printto") == 0) 
				continue;
			// If menu item has a default value, that is the true menu text
			{
				HKEY hSubKey;

				if (RegOpenKeyEx (hKey, szMenu, 0, (REGSAM)KEY_READ, &hSubKey) == ERROR_SUCCESS) {

					lSize = sizeof (szItem);
					lRC = RegQueryValueEx (hSubKey, "", NULL, &lType, (LPBYTE)szItem, &lSize);
					// If no (or empty) default text, use keyname as menu text.
					if ((lRC != ERROR_SUCCESS) || (lSize < 2))
						lstrcpy (szItem, szMenu);
					RegCloseKey (hSubKey);
				} else
					lstrcpy (szItem, szMenu);
			}

			// See if we need to tag item as the default.
			// DefSet = 0, no default set.
			// DefSet = 1, menu title text indicating default
			// DefSet = 2, menu verb indicating default.

			if ((sDefSet < 2) && (lstrcmpi (szMenu, szDefault) == 0)) {
				sDefSet = 2;
				sDefItem = sListBoxCnt;
			} else if ((sDefSet < 1) && (lstrcmpi (szItem, szDefault) == 0)) {
				sDefSet = 1;
				sDefItem = sListBoxCnt;
			}
			lstrcat (szItem, "\t\t");
			// Append key value so we can find command easily.
			lstrcat (szItem, lpszKey);
			lstrcat (szItem, "\\");
			lstrcat (szItem, szMenu);

			// ListBoxCnt is needed because the PrintTo item isn't added so 'i' is
			// not hte number of items in the listbox.
			sListBoxCnt++; 
			SendMessage (hWnd, LB_ADDSTRING, 0, (LPARAM)(LPSTR)szItem);
		}
		RegCloseKey (hKey);

		// If a default item wasn't explicitly set, mark top item as default.
		if (sDefSet == 0)
			sDefItem = 0;
		// Don't mark a default item for All Files.
		if (lstrcmp (szFileDesc, "All Files") == 0)
			sDefItem = -1;
		// Now go back and mark default menu item.
		i = SendMessage (hWnd, LB_GETTEXT, sDefItem, (LPARAM)(LPSTR)szMenu);
		if (i != LB_ERR) {
			// Find first tab
			for (i = 0; szMenu[i] && (szMenu[i] != '\t'); i++);
			// Insert default string
			if (szMenu[i]) {
				lstrcpyn (szItem, szMenu, ++i);
				lstrcat (szItem, "\t(Default)");
				lstrcat (szItem, &szMenu[i]);
			}
			SendMessage (hWnd, LB_DELETESTRING, sDefItem, 0);
			SendMessage (hWnd, LB_INSERTSTRING, 0, (LPARAM)(LPSTR)szItem);
		}
	}
	return 0;
}
//------------------------------------------------------------
// QueryKey - Helper routine for FillAddEditStruct
//------------------------------------------------------------
INT QueryKey (char *szKey, char *szItem, LONG lItemSize) {
	HKEY hKey;
	LONG lType, lRC;

	// If menu item has a default value, that is the true menu text
	if (RegOpenKeyEx (HKEY_CLASSES_ROOT, szKey, 0, (REGSAM)KEY_ALL_ACCESS, &hKey) == ERROR_SUCCESS) {

		lRC = RegQueryValueEx (hKey, "", NULL, &lType, (LPBYTE)szItem, &lItemSize);
		// If no (or empty) default text, use keyname as menu text.
		if ((lRC != ERROR_SUCCESS) || (lItemSize < 2))
			szItem[0] = '\0';

		RegCloseKey (hKey);
	} else
		szItem[0] = '\0';

	return lstrlen (szItem);
}
//------------------------------------------------------------
// FillAddEditStruct - Gets verb structure data from the reg.
// Assume that the structure was inited with zeros elsewhere.
//------------------------------------------------------------
INT FillAddEditStruct (char *szKey, LPADDEDITSTRUCT lpAES) {
	INT i;
	char *pszTemp;
	char szTemp[256], *pszEnd;

	pszEnd = szKey + lstrlen (szKey);
	// Copy menu item name
	pszTemp = strrchr (szKey, '\\');
	if (pszTemp)
		lstrcpy (lpAES->szAction, pszTemp+1);
	else
		return -1;
	
	// If menu name has a default value, its the true menu name.
	i = QueryKey (szKey, szTemp, sizeof (szTemp));
	if (i)
		lstrcpy (lpAES->szAction, szTemp);

	// Get command.
	lstrcat (szKey, "\\command");
	i = QueryKey (szKey, lpAES->szCommand, sizeof (lpAES->szCommand));
	if (i == 0)
		lpAES->szCommand[0] = '\0';

	// Get DDE exec if available
	lstrcpy (strrchr (szKey, '\\'), "\\ddeexec");
	i = QueryKey (szKey, lpAES->szDDEMsg, sizeof (lpAES->szDDEMsg));
	if (i) {
		lstrcat (szKey, "\\application");
		i = QueryKey (szKey, lpAES->szDDEApp, sizeof (lpAES->szDDEApp));
		if (i == 0)
			lpAES->szDDEApp[0] = '\0';

		lstrcpy (strrchr (szKey, '\\'), "\\ifexec");
		i = QueryKey (szKey, lpAES->szDDEAppNR, sizeof (lpAES->szDDEAppNR));
		if (i == 0)
			lpAES->szDDEAppNR[0] = '\0';

		lstrcpy (strrchr (szKey, '\\'), "\\topic");
		i = QueryKey (szKey, lpAES->szDDETopic, sizeof (lpAES->szDDETopic));
		if (i == 0)
			lpAES->szDDETopic[0] = '\0';
		lpAES->fUseDDE = TRUE;
	} else {
		lpAES->fUseDDE = FALSE;
	}
	*pszEnd = '\0';
	return 0;
}
//------------------------------------------------------------
// SaveMenuRegStuff - Writes the verb info to the registry
//------------------------------------------------------------
INT SaveMenuRegStuff (char *szKey, LPADDEDITSTRUCT lpAES, BOOL fNew) {
	BOOL fNormal;
	INT i, sLen, rc = 0;
	HKEY hKey, hSubKey;
	LONG lRC;
	char szTemp[256], *pszTemp, *pszEnd;

	// Delete any values already set.
	if (!fNew) {
		pszTemp = strrchr (szKey, '\\');
		if (pszTemp == 0)
			return -1;
		lRC = RegDeleteKey (HKEY_CLASSES_ROOT, szKey);
		if (lRC != ERROR_SUCCESS)
			return -1;
		*pszTemp = '\0';
	}

	// Allow only keys of 32 or less
	sLen = lstrlen (lpAES->szAction);
	if (sLen > 32) {
		fNormal = FALSE;
		sLen = 32;
	} else
	fNormal = TRUE;
	// Make sure only allowable chars are in key.
	lstrcpy (szTemp, lpAES->szAction);
	for (i = 0; (i < sLen) && lpAES->szAction[i]; i++) {
		if (lpAES->szAction[i] == '&')
			;
		else if (!IsCharAlpha (lpAES->szAction[i])) {
			lpAES->szAction[i] = '_';
			fNormal = FALSE;
		}
	}
	lpAES->szAction[i] = '\0';

	// Open the key.
	lstrcat (szKey, "\\");
	lstrcat (szKey, lpAES->szAction);
	pszEnd = szKey + lstrlen (szKey);

	lRC = RegCreateKey (HKEY_CLASSES_ROOT, szKey, &hKey);
	if (lRC != ERROR_SUCCESS)
		return -1;

	// If the menu name is too long or has strange chars, 
	// add a default value that will become the menu name.
	if (!fNormal) 
		RegSetValue (HKEY_CLASSES_ROOT, szKey, REG_SZ,
		             szTemp, lstrlen (szTemp));

	lRC = RegSetValue (hKey, "command", REG_SZ,	lpAES->szCommand, 
					   lstrlen (lpAES->szCommand));
	if (lRC != ERROR_SUCCESS) {
		RegCloseKey (hKey);
		return -2;
	}

	// Add DDE Exec strings if necessary
	if (lpAES->fUseDDE) {
		lstrcpy (pszEnd, "\\");
		lstrcat (pszEnd, "ddeexec");

		lRC = RegSetValue (hKey, "ddeexec", REG_SZ,	lpAES->szDDEMsg, 
						   lstrlen (lpAES->szDDEMsg));
		if (lRC != ERROR_SUCCESS) {
			RegCloseKey (hKey);
			return -3;
		}
		lRC = RegOpenKey (hKey, "ddeexec", &hSubKey);
		if (lRC == ERROR_SUCCESS) {
			if (lstrlen (lpAES->szDDEApp)) {
				lRC = RegSetValue (hSubKey, "application", REG_SZ, lpAES->szDDEApp, 
								   lstrlen (lpAES->szDDEApp));
				if (lRC != ERROR_SUCCESS) 
					rc = -5;
			}
			if ((rc == 0) && (lstrlen (lpAES->szDDEAppNR))) {
				lRC = RegSetValue (hSubKey, "ifexec", REG_SZ, lpAES->szDDEAppNR,
								   lstrlen (lpAES->szDDEAppNR));
				if (lRC != ERROR_SUCCESS) 
					rc = -6;
			}
			if ((rc == 0) && (lstrlen (lpAES->szDDETopic))) {
				lRC = RegSetValue (hSubKey, "topic", REG_SZ, lpAES->szDDETopic, 
								   lstrlen (lpAES->szDDETopic));
				if (lRC != ERROR_SUCCESS) 
					rc = -7;
			}
			RegCloseKey (hSubKey);
		} else
			rc = -3;
	}
	RegCloseKey (hKey);
	return rc;
}
//============================================================
// AddEditDlgProc - AddEdit dialog box dialog procedure
//============================================================
BOOL CALLBACK AddEditDlgProc (HWND hWnd, UINT wMsg, UINT wParam, 
                              LONG lParam) {

	static LPADDEDITSTRUCT lpAES;
	char szTemp[256];

	switch (wMsg) {                              
		case WM_INITDIALOG:

			if (lParam) 
				lpAES = (LPADDEDITSTRUCT)lParam;
			else {
				EndDialog(hWnd, 0);
				return TRUE;
			}
			SendDlgItemMessage (hWnd, IDD_ACTION, EM_LIMITTEXT, 40, 0);
			SendDlgItemMessage (hWnd, IDD_COMMAND, EM_LIMITTEXT, 255, 0);
			SendDlgItemMessage (hWnd, IDD_DDEMSG, EM_LIMITTEXT, 255, 0);
			SendDlgItemMessage (hWnd, IDD_DDEAPP, EM_LIMITTEXT, 255, 0);
			SendDlgItemMessage (hWnd, IDD_DDEAPPNR, EM_LIMITTEXT, 255, 0);
			SendDlgItemMessage (hWnd, IDD_DDETOPIC, EM_LIMITTEXT, 255, 0);
		
			SetDlgItemText (hWnd, IDD_ACTION, lpAES->szAction);
			SetDlgItemText (hWnd, IDD_COMMAND, lpAES->szCommand);
			// Get DDE exec if available
			SetDlgItemText (hWnd, IDD_DDEMSG, lpAES->szDDEMsg);
			SetDlgItemText (hWnd, IDD_DDEAPP, lpAES->szDDEApp);
			SetDlgItemText (hWnd, IDD_DDEAPPNR, lpAES->szDDEAppNR);
			SetDlgItemText (hWnd, IDD_DDETOPIC, lpAES->szDDETopic);

			if (lpAES->fUseDDE)
				CheckDlgButton (hWnd, IDD_USEDDE, BST_CHECKED);
			else {
				SetDlgItemText (hWnd, IDD_DDEMSG, "");
				SetDlgItemText (hWnd, IDD_DDEAPP, "");
				SetDlgItemText (hWnd, IDD_DDEAPPNR, "");
				SetDlgItemText (hWnd, IDD_DDETOPIC, "");
				EnableWindow (GetDlgItem (hWnd, IDD_DDEMSG), FALSE);
				EnableWindow (GetDlgItem (hWnd, IDD_DDEMSGTEXT), FALSE);
				EnableWindow (GetDlgItem (hWnd, IDD_DDEAPP), FALSE);
				EnableWindow (GetDlgItem (hWnd, IDD_DDEAPPTEXT), FALSE);
				EnableWindow (GetDlgItem (hWnd, IDD_DDEAPPNR), FALSE);
				EnableWindow (GetDlgItem (hWnd, IDD_DDEAPPNRTEXT), FALSE);
				EnableWindow (GetDlgItem (hWnd, IDD_DDETOPIC), FALSE);
				EnableWindow (GetDlgItem (hWnd, IDD_DDETOPICTEXT), FALSE);
				CheckDlgButton (hWnd, IDD_USEDDE, BST_UNCHECKED);
			}
			// If this is a new item, set the focus to the menu name.  Otherwise,
			// set the focus to the command.
			if (lstrlen (lpAES->szAction) == 0) {
				lstrcpy (szTemp, "Add action for ");
				lstrcat (szTemp, lpAES->szFileType);
				SetWindowText (hWnd, szTemp);
				SetFocus (GetDlgItem (hWnd, IDD_ACTION));
			} else {
				lstrcpy (szTemp, "Edit action for ");
				lstrcat (szTemp, lpAES->szFileType);
				SetWindowText (hWnd, szTemp);
				SetFocus (GetDlgItem (hWnd, IDD_COMMAND));
			}
			return FALSE;  // Return false to avoid auto set of active ctl.
	
		case WM_COMMAND:
			{
				UINT idItem, wNotifyCode;

				#if (WIN32)
					idItem = (UINT) LOWORD(wParam);
					wNotifyCode = (UINT) HIWORD(wParam);
				#else
					idItem = (UINT) wParam;
					wNotifyCode = (UINT) HIWORD(lParam);
				#endif

				switch (idItem) {

					case IDD_USEDDE:
						if (wNotifyCode == BN_CLICKED) {
							BOOL fEnable;

							if (IsDlgButtonChecked (hWnd, IDD_USEDDE) == 0)
								fEnable = FALSE;
							else
								fEnable = TRUE;

							EnableWindow (GetDlgItem (hWnd, IDD_DDEMSG), fEnable);
							EnableWindow (GetDlgItem (hWnd, IDD_DDEMSGTEXT), fEnable);
							EnableWindow (GetDlgItem (hWnd, IDD_DDEAPP), fEnable);
							EnableWindow (GetDlgItem (hWnd, IDD_DDEAPPTEXT), fEnable);
							EnableWindow (GetDlgItem (hWnd, IDD_DDEAPPNR), fEnable);
							EnableWindow (GetDlgItem (hWnd, IDD_DDEAPPNRTEXT), fEnable);
							EnableWindow (GetDlgItem (hWnd, IDD_DDETOPIC), fEnable);
							EnableWindow (GetDlgItem (hWnd, IDD_DDETOPICTEXT), fEnable);
						}
						break;

					case IDD_BROWSE:
						{

							static char szFilter[] = {"Programs (*.EXE;*.COM;*.PIF;*.BAT)\0*.EXE;*.COM;*.PIF;*.BAT\0"\
							                          "All Files (*.*)\0*.*\0"\
							                          "\0\0"};
							char szStr[256];

							szStr[0] = '\0';
							if (MyGetFilename (hWnd, szStr, sizeof (szStr), 
								               OFN_FILEMUSTEXIST | OFN_HIDEREADONLY, szFilter, 1) == 0)
								break;
							SetDlgItemText (hWnd, IDD_COMMAND, szStr);
						}
						break;

					case IDD_TIPS:
						{
							MessageBox (hWnd, "The following parameters work on the command line:\n"\
							            "%1 - Name of the file\n%2 - Name of the default printer.",
										"Tips", MB_OK | MB_ICONINFORMATION);
						}
						break;

					case IDOK:
						{
							char *pszTemp;
							INT rc = 0;

							// Verify that the action doesn't conflict with other
							// menu items.
							GetDlgItemText (hWnd, IDD_ACTION, lpAES->szAction, 
								            sizeof (lpAES->szAction));
							if (lstrlen (lpAES->szAction)) {
								pszTemp = lpAES->szOthers;
								while (*pszTemp) {
									if (lstrcmpi (lpAES->szAction, pszTemp) == 0) {
										MessageBox (hWnd, "The action text matches another menu item.",
										            "Problem with menu text", MB_OK | MB_ICONSTOP);
										rc = -2;
										break;
									} else
										pszTemp += lstrlen (pszTemp) + 1;
								}
							} else {
								MessageBox (hWnd, "You must specify action text.",
								            "Problem with menu text", MB_OK | MB_ICONSTOP);
								rc = -1;
							}
							GetDlgItemText (hWnd, IDD_COMMAND, lpAES->szCommand, 
								            sizeof (lpAES->szCommand));

							// See if we have command text
							if (lstrlen (lpAES->szCommand) == 0) {
								MessageBox (hWnd, "You must specify a command.",
								            "Problem with command text", MB_OK | MB_ICONSTOP);
								rc = -3;
							}
							if (rc == 0) {
								GetDlgItemText (hWnd, IDD_DDEMSG, lpAES->szDDEMsg, 
									            sizeof (lpAES->szDDEMsg));
								GetDlgItemText (hWnd, IDD_DDEAPP, lpAES->szDDEApp, 
									            sizeof (lpAES->szDDEApp));
								GetDlgItemText (hWnd, IDD_DDEAPPNR, lpAES->szDDEAppNR, 
									            sizeof (lpAES->szDDEAppNR));
								GetDlgItemText (hWnd, IDD_DDETOPIC, lpAES->szDDETopic, 
									            sizeof (lpAES->szDDETopic));
								if (IsDlgButtonChecked (hWnd, IDD_USEDDE) == 0)
									lpAES->fUseDDE = FALSE;
								else
									lpAES->fUseDDE = TRUE;
								EndDialog(hWnd, 1);
							}
						}
						return TRUE;

					case IDCANCEL:
						EndDialog(hWnd, 0);
						return TRUE;
				}
			}
			break;
	} 
	return FALSE;
}

//============================================================  
// General Helper Routines 
//============================================================  
//------------------------------------------------------------ 
// ParseAboutStr - Parses about box string to include version
// information.
// Note: Strings enclosed in percent signs are replaced with
//       version info from the rc file.  
//       ~O is replaced with the current operating system
//       ~T is replaced with the target operating system
//       ~H is replaced with the target hardware
//------------------------------------------------------------ 
INT ParseAboutString (LPVOID lpVerData, LPSTR lpszKey, 
                      LPSTR lpszIn, LPSTR lpszOut, UINT usMax) {
	UINT usLen;
	LPSTR lpszKeyEnd, lpszTemp;
	LPSTR lpszData, lpszStart;
	char szTemp[80];
	DWORD dwData;
	
	lpszKeyEnd = lpszKey + lstrlen (lpszKey);
	lpszStart = lpszOut;

	while (*lpszIn && usMax) {
		// See if start of version info key
		if (*lpszIn == '%') {
			lpszIn++;      //Skip past initial %
			if (*lpszIn != '%') {
			
				lpszTemp = lpszKeyEnd;
				// Copy key
				while (*lpszIn && (*lpszIn != '%')) 
					*lpszTemp++ = *lpszIn++;
				*lpszTemp = '\0';
			
				VerQueryValue (lpVerData, lpszKey, &lpszData, &usLen);
				*lpszKeyEnd = '\0';
				if (usLen && (usLen < usMax)) {
					lstrcpy (lpszOut, lpszData);
					usLen--;   //Length includes terminating zero we don't want
					lpszOut += usLen;
					usMax -= usLen;
				}	
			} else {
				*lpszOut++ = '%';
				usMax--;
			}
			lpszIn++;      //Skip past trailing %

		} else if (*lpszIn == '~') {
			lpszIn++;      //Skip past initial %
			szTemp[0] = *lpszIn;
			szTemp[1] = '\0';
			switch (*lpszIn) {

				// Hardware platform
				case 'H':
				case 'h':
#if (WIN32)
					{
						SYSTEM_INFO si;
						static char *pszIntel[] = {"386","486",
						                           "Pentium","P6"};
						memset (&si, 0, sizeof (si));
						GetSystemInfo (&si);
						dwData = GetVersion ();
						switch (si.wProcessorArchitecture) {
							case PROCESSOR_ARCHITECTURE_INTEL:
								// If Win95, don't use full structure
								if ((dwData & 0x80000000) && 
								    (LOBYTE (LOWORD (dwData)) >= 4)) {
									wsprintf (szTemp, "Intel %d", si.dwProcessorType);
								} else {
									lstrcpy (szTemp, "Intel ");
									if ((si.wProcessorLevel > 2) && 
										 (si.wProcessorLevel < 7)) 
										lstrcat (szTemp, 
										         pszIntel[si.wProcessorLevel-3]);
									else				
										lstrcat (szTemp, "Microprocessor ");

									wsprintf (&szTemp[lstrlen(szTemp)], " %c%d", 
									          (si.wProcessorRevision >> 16) + 'A',
									          si.wProcessorRevision && 0xff);
								}
								break;

							case PROCESSOR_ARCHITECTURE_MIPS:
								wsprintf	(szTemp, "MIPS R%d", 
											 si.wProcessorLevel + 1000);
								break;

							case PROCESSOR_ARCHITECTURE_ALPHA:
								wsprintf	(szTemp, "DEC Alpha %d Model %d Pass %d", 
											 si.wProcessorLevel,
											 (si.wProcessorRevision >> 16) + 'A',
											 si.wProcessorRevision && 0xff);
								break;

							case PROCESSOR_ARCHITECTURE_PPC:
								lstrcpy (szTemp, "PowerPC  ");
								switch (si.wProcessorLevel) {
									case 1:
										lstrcat (szTemp, "601");
										break;
									case 3:
										lstrcat (szTemp, "603");
										break;
									case 4:
										lstrcat (szTemp, "604");
										break;
									case 6:
										lstrcat (szTemp, "603+");
										break;
									case 9:
										lstrcat (szTemp, "604+");
										break;
									case 20:
										lstrcat (szTemp, "620");
										break;
								}
								wsprintf	(&szTemp[lstrlen(szTemp)], " %d.%d", 
											 (si.wProcessorRevision >> 16),
											 si.wProcessorRevision && 0xff);
								break;
						}
					}	
#else					
					dwData = GetWinFlags ();
					lstrcpy (szTemp, "Intel ");
					if (dwData & WF_CPU386)
						lstrcat (szTemp, "386");
					else if (dwData & WF_CPU486)
						lstrcat (szTemp, "486");
					else 
						lstrcat (szTemp, "Microprocessor");
#endif
					break;

				// Target OS
				case 'T':
				case 't':
#if (WIN32)
#if (WINVER < 0x400)
					lstrcpy (szTemp, "Windows NT");
#else					
					lstrcpy (szTemp, "Windows 95");
#endif
#else
					lstrcpy (szTemp, "Windows 3.1");
#endif
					break;

				// Current OS
				case 'O':
				case 'o':
					dwData = GetVersion();
					// See if Win NT (high bit 0)
#if (WIN32)						
					if (dwData < 0x80000000) {
						wsprintf (szTemp, "Windows NT %u.%u (Build: %u)",
						          (DWORD)(LOBYTE(LOWORD(dwData))),
						          (DWORD)(HIBYTE(LOWORD(dwData))),
						          (DWORD)HIWORD(dwData));

					} else if (LOBYTE (LOWORD (dwData) < 4)) {
						wsprintf (szTemp, "Win32s %u.%u (Build: %u)",
						          (DWORD)(LOBYTE(LOWORD(dwData))),
						          (DWORD)(HIBYTE(LOWORD(dwData))),
						          (DWORD)(HIWORD(dwData) & 0x7fff));
					} else {
						wsprintf (szTemp, "Windows 95 %u.%u",
						          (DWORD)(LOBYTE(LOWORD(dwData))),
						          (DWORD)(HIBYTE(LOWORD(dwData))));
					}
#else					
					if (GetWinFlags () & 0x4000) 
						lstrcpy (szTemp, "Windows NT WOW");
					else {
						if ((LOBYTE(LOWORD(dwData)) == 3) &&
						    (HIBYTE(LOWORD(dwData)) >= 95))
							wsprintf (szTemp, "Windows %u",
							          HIBYTE(LOWORD(dwData)));
						else
							wsprintf (szTemp, "Windows %u.%u  DOS %u.%u",
							          LOBYTE(LOWORD(dwData)),
							          HIBYTE(LOWORD(dwData)),
							          HIBYTE(HIWORD(dwData)),
							          LOBYTE(HIWORD(dwData)));
					}
#endif
					break;
			}					
			lpszIn++;
			usLen = lstrlen (szTemp);
			if (usLen && (usLen < usMax)) {
				lstrcpy (lpszOut, szTemp);
				lpszOut += usLen;
				usMax -= usLen;
			}
		} else {
			*lpszOut++ = *lpszIn++;
			usMax--;
		}
	}	
	*lpszOut = '\0';
	return lstrlen (lpszStart);
}	
//============================================================
// AboutDlgProc - About dialog box dialog procedure
//============================================================
BOOL CALLBACK AboutDlgProc (HWND hWnd, UINT wMsg, UINT wParam, 
                            LONG lParam) {
	RECT rect;
	HDC hdc;
	PAINTSTRUCT ps;
	HPEN hDPen, hLPen, hOldPen;
	char szIn[256], szOut[256], szKey[256];
	char szThisFile[MAX_PATH];
	LPSTR lpVerData;
	DWORD dwSize, dwHandle;
	HGLOBAL hMem;
	HWND hCtl;
	INT i;

	switch (wMsg) {                              
		case WM_INITDIALOG:
			// Read version info from file
			GetModuleFileName (hInst, szThisFile, sizeof (szThisFile));
			dwSize = GetFileVersionInfoSize (szThisFile, &dwHandle);		
			if (dwSize) {			
				hMem = GlobalAlloc (GMEM_MOVEABLE, dwSize);
				lpVerData = GlobalLock (hMem);
				if (lpVerData) {
					GetFileVersionInfo (szThisFile, dwHandle, dwSize, lpVerData);

					// Create ptr to version info resource (American English)
					lstrcpy (szKey, "\\StringFileInfo\\040904E4\\");

					// Set About box title
					GetWindowText (hWnd, szIn, sizeof (szIn));
					ParseAboutString ((LPVOID)lpVerData, szKey, szIn,
					                  szOut, sizeof (szOut));
					SetWindowText (hWnd, szOut);		

					// Get and set other about box fields
					for (i = IDD_ABOUT1; i < IDD_ABOUTLAST; i++) {
						hCtl = GetDlgItem (hWnd, i);
						if (hCtl) {						  
							GetWindowText (hCtl, szIn, sizeof (szIn)); 
							ParseAboutString ((LPVOID)lpVerData, szKey, szIn,
							                  szOut, sizeof (szOut));
							SetWindowText (hCtl, szOut);		
						}
					}
					GlobalUnlock (hMem);
					GlobalFree (hMem);
				}				

			}		
			return TRUE;

#if (WIN16)
		case WM_CTLCOLOR:
			SetBkColor ((HDC) wParam, RGB (192, 192, 192));
			return (BOOL)GetStockObject (LTGRAY_BRUSH);
#endif		
	
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			hDPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNSHADOW));
			hLPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNHIGHLIGHT));

			for (i = IDD_ALINE1; i < IDD_ALINELAST; i++) {
				hCtl = GetDlgItem (hWnd, i);
				if (IsWindow (hCtl)) {
					// Use pos of control to place line
					GetClientRect (hCtl, &rect);
					ClientToScreen (hCtl, (LPPOINT)&rect);
					ScreenToClient (hWnd, (LPPOINT)&rect);
					rect.top -= 2;
					rect.right += rect.left;
					// Draw dark then light line to get 3d effect
					hOldPen = SelectObject (hdc, hDPen);
					MoveTo (hdc, rect.left, rect.top);
					LineTo (hdc, rect.right, rect.top);
					SelectObject (hdc, hLPen);
					MoveTo (hdc, rect.left, rect.top+1);
					LineTo (hdc, rect.right, rect.top+1);
					SelectObject (hdc, hOldPen);
				} else
					break;
			}	
			DeleteObject (hDPen);
			DeleteObject (hLPen);
			EndPaint(hWnd, &ps);
			return TRUE;
	
		case WM_COMMAND:
		   if ((wParam == IDOK) || (wParam == IDCANCEL)) {
				EndDialog(hWnd, 0);
				return TRUE;
			} 
	} 
	return FALSE;
}
//------------------------------------------------------------
// MyDisplayDialog - Display a dialog box
//------------------------------------------------------------ 
INT MyDisplayDialog (HINSTANCE hInstance, LPCSTR szDlgName,
                     HWND hWnd, WNDPROC lpDialogProc, 
                     LPARAM lParam) {
    WNDPROC lpDlgProcInst;
    INT		rc;

    lpDlgProcInst = MakeProcInstance(lpDialogProc, hInst);
    rc = DialogBoxParam (hInstance, szDlgName, hWnd, 
                         lpDlgProcInst, lParam);
    FreeProcInstance(lpDlgProcInst);
    return rc;                              
}
//------------------------------------------------------------
// MyWritePrivateProfileInt - Writes an integer to the profile
//------------------------------------------------------------
BOOL MyWritePrivateProfileInt (char *szSec, char *szEntry, 
                               int Num, char *szProfile) {
	char	szStr[33];
	                           
	wsprintf (szStr, "%d", Num);
	return WritePrivateProfileString (szSec, szEntry, szStr, 
	                                  szProfile);
}

//------------------------------------------------------------
// PostWm_CommandMsg - Posts a MW_COMMAND message
// This is necessary since Win16 and Win32 formats different
//------------------------------------------------------------
INT PostWm_CommandMsg (HWND hWnd, WORD idItem, WORD wNotifyCode, HWND hCtl) {
	LONG lParam;
	
#if (WIN32)
	LONG wParam;

	wParam = MAKELPARAM (idItem, wNotifyCode);
	lParam = (LPARAM) hCtl;
#else
	UINT wParam;

	wParam = idItem;
	lParam = MAKELPARAM (hCtl, wNotifyCode);
#endif

	PostMessage (hWnd, WM_COMMAND, wParam, lParam);
	return 0;
}

//------------------------------------------------------------
// MyGetFilename - Uses the common File Open dialog box to
// query the user for a filename
//------------------------------------------------------------
BOOL MyGetFilename (HWND hWnd, LPSTR szFilename, INT sMaxLen, DWORD fFlags,
                    LPSTR szFilter, INT sFilterIndex) {
   OPENFILENAME ofn;
	INT rc, i;
 
    ofn.lStructSize       = sizeof(ofn); 
    ofn.hwndOwner         = hWnd; 
    ofn.hInstance         = (HANDLE) hInst; 
    ofn.lpstrFilter       = szFilter; 
    ofn.lpstrCustomFilter = (LPTSTR) NULL; 
    ofn.nMaxCustFilter    = 0L; 
    ofn.nFilterIndex      = sFilterIndex; 

    ofn.lpstrFile         = szFilename; 
    ofn.nMaxFile          = sMaxLen; 

    ofn.lpstrFileTitle    = 0; 
    ofn.nMaxFileTitle     = 0;//sizeof (szFileTitle); 

    ofn.lpstrFileTitle    = 0;
    ofn.nMaxFileTitle     = 0; 

    ofn.lpstrInitialDir   = NULL; 
    ofn.lpstrTitle        = TEXT("Open with"); 
    ofn.nFileOffset       = 0; 
    ofn.nFileExtension    = 0; 
    ofn.lpstrDefExt       = 0; 
    ofn.lCustData         = 0; 
	ofn.Flags             = fFlags;

 	rc = 0;
	i = GetOpenFileName (&ofn);
	if (i)
		rc++;
		if (ofn.Flags & OFN_READONLY)
			rc++;
 	return rc;
}
//------------------------------------------------------------
// MyRegisterClass - Win16/Win32 generic class reg routine
//------------------------------------------------------------
ATOM MyRegisterClass (WNDCLASS *pwc) {

#if (WIN32)	
	HINSTANCE hUser;
	FARPROC lpfnRCEx;
	WNDCLASSEX wcx;
	
	hUser = GetModuleHandle ("USER32");
	if (hUser) {

#if defined (UNICODE)
		lpfnRCEx = GetProcAddress (hUser, "RegisterClassExW");
#else	
		lpfnRCEx = GetProcAddress (hUser, "RegisterClassExA");
#endif	
		// This function new for Win95
		if (lpfnRCEx) {
			// Copy wndclass data from old structure
			wcx.style = pwc->style;
			wcx.lpfnWndProc = pwc->lpfnWndProc;
			wcx.cbClsExtra = pwc->cbClsExtra;
			wcx.cbWndExtra = pwc->cbWndExtra;
			wcx.hInstance = pwc->hInstance;
			wcx.hIcon = pwc->hIcon;
			wcx.hCursor = pwc->hCursor;
			wcx.hbrBackground = pwc->hbrBackground;
			wcx.lpszMenuName = pwc->lpszMenuName;
			wcx.lpszClassName = pwc->lpszClassName;
			// Add new stuff
			wcx.cbSize = sizeof (WNDCLASSEX);
			wcx.hIconSm = LoadIcon (pwc->hInstance, "SMALL");

			return (*lpfnRCEx)(&wcx);
		} else
			return RegisterClass(pwc);
	}
#endif	
	return RegisterClass(pwc);
}
//------------------------------------------------------------
// MySubClassWindow - Subclasses a window 
//------------------------------------------------------------
WNDPROC MySubClassWindow (HWND hWnd, WNDPROC lpfnNewProc) {
   WNDPROC lpfnOldProc;

	lpfnOldProc = (WNDPROC) GetWindowLong (hWnd, GWL_WNDPROC);
	SetWindowLong (hWnd, GWL_WNDPROC, (LONG) lpfnNewProc);
	return lpfnOldProc;				               
}				               
//------------------------------------------------------------
// MyYield - Yields control to other programs, but returns
// if Windows is idle.
//------------------------------------------------------------
BOOL MyYield (void) {
   MSG	msg;
   BOOL	bCont;
   
   bCont = TRUE;
	while (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE)) {
	   if (msg.message == WM_QUIT)
	      bCont = FALSE;
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return bCont;
}
//------------------------------------------------------------
// GetErrorString - Returns an error string 
//------------------------------------------------------------ 
BOOL GetErrorString (INT sErrorCode, char *pszOut, INT sMax) {

	lstrcpy (pszOut, "Error - ");
   sErrorCode = abs (sErrorCode);
	if (LoadString (hInst, sErrorCode, pszOut + lstrlen(pszOut), 
	                sMax - lstrlen (pszOut)) == 0) 
		wsprintf (pszOut, "Error number: %d", sErrorCode);

	return TRUE;
}	                    
//------------------------------------------------------------
// PrintError - Displays a message box with an error code
//------------------------------------------------------------ 
void PrintError (HWND hWnd, INT rc) {
	char szErrStr[80];

	GetErrorString (rc, szErrStr, sizeof (szErrStr));
	MessageBox (hWnd, szErrStr, szAppName, MB_ICONSTOP | MB_OK);
	return;
}
//------------------------------------------------------------
// MyCtl3dStuff - Bundled calls to use 3d effects under Win16
//------------------------------------------------------------ 
INT MyCtl3dStuff (HINSTANCE *phLib, HINSTANCE hMe, INT sFunction) {

#if (WIN16)
	FARPROC lp3dproc;
	
	switch (sFunction) {


		case CTL3D_INIT:
			// If 3DLib present, load it for 3d effects.
			*phLib = LoadLibrary ("CTL3D.DLL");
			if (*phLib < (HINSTANCE) HINSTANCE_ERROR)
				*phLib = 0;
			else {
				lp3dproc = GetProcAddress (*phLib, "Ctl3dRegister");
				if (lp3dproc) {
					(*lp3dproc) (hMe);

					lp3dproc = GetProcAddress (*phLib, "Ctl3dAutoSubclass");
					if (lp3dproc) {
						(*lp3dproc) (hMe);
					}	

				} else {
					FreeLibrary (*phLib);
					*phLib = 0;
				}	
			}	
			break;

		case CTL3D_TERM:
			// Unregister 3d-lib
			if (*phLib) {
				lp3dproc = GetProcAddress (*phLib, "Ctl3dUnregister");
				if (lp3dproc) 
					(*lp3dproc) (hMe);
				FreeLibrary (*phLib);
			}

	}	
#endif
	return 0;
}	
